# Copyright (C) 1999, 2000 Jay Beale
# assisted by Peter Watkins
# Licensed under the GNU General Public License

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#
# NOTE   : This module is a stand-in for a more featureful model that uses all
#          the functionality of Peter's new firewall script.  Basically, this
#          doesn't handle dhcp-type connections nearly as well.  This is JJB's
#          temporary stand-in for a more featureful version...
#

package Bastille::IPCHAINS;

use Bastille::API;
@ENV="";
undef(%ENV);
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";

# @ipchainsOptions contains information about the variables this script sets the value of
# in the final firewall script. Each item in @ipchainsOptions is a hash describing a certain
# environment variable. 
#
# Each such hash should have the following name-value pairs:
#	varname		the actual variable name for the /bin/sh firewall script
#	default		the default/suggested value for that variable (in case nothing in the config)
#	stanza		where this is used in the template script
#	configname	the name for the user's choice for this question, as read
#			from the "config" script


@ipchainsOptions = (

	{
	'varname' => "DNS_SERVERS",
	'default' => "",
	'stanza' => "0",
	'configname' => 'ip_s_dns',
	},

	{
	'varname' => "TRUSTED_IFACES",
	'default' => "lo",
	'stanza' => "1",
	'configname' => 'ip_s_trustiface',
	},

	{
	'varname' => "PUBLIC_IFACES",
	'default' => "eth+ ppp+",
	'stanza' => "1",
	'configname' => 'ip_s_publiciface',
	},

	{
	'varname' => "INTERNAL_IFACES",
	'default' => "",
	'stanza' => "1",
	'configname' => 'ip_s_internaliface',
	},

	{
	'varname' => "TCP_AUDIT_SERVICES",
	'default' => "telnet ftp imap pop-3 finger sunrpc exec login linuxconf ssh",
	'stanza' => "2",
	'configname' => 'ip_s_tcpaudit',
	},

	{
	'varname' => "UDP_AUDIT_SERVICES",
	'default' => "31337",
	'stanza' => "2",
	'configname' => 'ip_s_udpaudit',
	},

	{
	'varname' => "ICMP_AUDIT_TYPES",
	'default' => "",
	'stanza' => "2",
	'configname' => 'ip_s_icmpaudit',
	},

	{
	'varname' => "TCP_PUBLIC_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_publictcp',
	},

	{
	'varname' => "UDP_PUBLIC_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_publicudp',
	},

	{
	'varname' => "TCP_INTERNAL_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_internaltcp',
	},

	{
	'varname' => "UDP_INTERNAL_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_internaludp',
	},

	{
	'varname' => "FORCE_PASV_FTP",
	'default' => "N",
	'stanza' => "4",
	'configname' => 'ip_s_passiveftp',
	},

	{
	'varname' => "TCP_BLOCKED_SERVICES",
	'default' => "6000:6020",
	'stanza' => "5",
	'configname' => 'ip_s_tcpblock',
	},

	{
	'varname' => "UDP_BLOCKED_SERVICES",
	'default' => "",
	'stanza' => "5",
	'configname' => 'ip_s_udpblock',
	},

	{
	'varname' => "ICMP_ALLOWED_TYPES",
	'default' => "destination-unreachable echo-reply time-exceeded",
	'stanza' => "5",
	'configname' => 'ip_s_icmpallowed',
	},

	{
	'varname' => "ENABLE_SRC_ADDR_VERIFY",
	'default' => "Y",
	'stanza' => "6",
	'configname' => 'ip_s_srcaddr',
	},

	{
	'varname' => "IP_MASQ_NETWORK",
	'default' => "",
	'stanza' => "7",
	'configname' => 'ip_s_ipmasq',
	},

	{
	'varname' => "IP_MASQ_MODULES",
	'default' => "ftp raudio vdolive",
	'stanza' => "7",
	'configname' => 'ip_s_kernelmasq',
	},

	{
	'varname' => "REJECT_METHOD",
	'default' => "DENY",
	'stanza' => "8",
	'configname' => 'ip_s_rejectmethod',
	},

	{
	'varname' => "DHCP_IFACES",
	'default' => "",
	'stanza' => "9",
	'configname' => 'ip_s_dhcpiface',
	},

	{
	'varname' => "NTP_SERVERS",
	'default' => "",
	'stanza' => "10",
	'configname' => 'ip_s_ntpsrv',
	},

	{
	'varname' => "ICMP_OUTBOUND_DISABLED_TYPES",
	'default' => "destination-unreachable time-exceeded",
	'stanza' => "11",
	'configname' => 'ip_s_icmpout',
	},

	);

# JJB's fix -- get this to work when they choose the ip_b_ questions instead.

print "About to run through config file...\n";

if ( $GLOBAL_CONFIG{IPChains}{ip_advnetwork} eq "N" ) {
    
    for (my $loop=0; $loop < scalar(@ipchainsOptions) ; ++$loop) {
	if ( $ipchainsOptions[$loop]{configname} =~ /^ip_s_(.*)$/ ) {
	    $ipchainsOptions[$loop]{configname} = "ip_b_" . $1;
	}
    }
}

# This is where the code uses Jay's subroutine, temporarily, to get this out the door faster...

&InstallScript_simpler;


sub ipchainsAddNetmask {
	# adds netmask info to plain dotted-quad IP addresses
	my ( $phrase ) = @_;
	if ( $phrase =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ ) {
		# looks like a plain IPv4 dot-quad address
		if ( $phrase eq '0.0.0.0' ) {
			$phrase .= '/0.0.0.0';
		} elsif ( $phrase =~ /\.0\.0\.0$/ ) {
			$phrase .= '/255.0.0.0';
		} elsif ( $phrase =~ /\.0\.0$/ ) {
			$phrase .= '/255.255.0.0';
		} elsif ( $phrase =~ /\.0$/ ) {
			$phrase .= '/255.255.255.0';
		} else {
			$phrase .= '/255.255.255.255';
		}
	} 
	return $phrase;
}

sub ipchainsCleanItemList {
	# removes spaces, cleans up whitespace in string
	my ( $list ) = @_;
	my ( @items, $one );
	$list =~ s/\,/ /g;
	$list =~ s/[^a-zA-Z0-9\-\_\s\.\:\+\/]//eg;	# remove unexpected chars
	$list =~ s/[\n\r\t]/ /g;
	while ( $list =~ /  / ) { $list =~ s/  / /g;}
	@items = split(/ /, $list);
	$list = '';
	foreach $one ( @items ) {
		$list .= ' '.&ipchainsAddNetmask($one);
	}
	$list =~ s/^.//;
	return $list;
}

sub InstallScript_simpler {

    # This is JJB's (possibly temporary) modification to IPCHAINS.pm to port to 1.1 architecture.

    my (@varArray,$loop);

    my $configPrefix="IPChains";

    my $firewall_init_script = $GLOBAL_DIR{"initd"} . "/bastille-firewall";
    my $virgin_init_script = "/bastille-firewall";

    # If the firewall script has not been installed, put it in place.
    unless ( -e $GLOBAL_PREFIX . $firewall_init_script ) {
	&B_place($virgin_init_script,$firewall_init_script);
	&B_chmod(0500,$firewall_init_script);
    }

    # Now, iterate through the answers filling in the script...
    for ($loop=0; $loop < scalar(@ipchainsOptions); ++$loop) {
	
	# Build the @varArray to hold text to be inserted.

	my $ans = &ipchainsCleanItemList($GLOBAL_CONFIG{$configPrefix}{$ipchainsOptions[$loop]{configname}});
	my $variable = $ipchainsOptions[$loop]{varname};
	my $assignment = $variable . "=\"$ans\"\n";

	my $pattern = "^${variable}\s*=";
	&B_replace_line($firewall_init_script,$pattern,$assignment);

    } 

    # Place the bastille-firewall-schedule file
    my $placed_file = $GLOBAL_DIR{"sbin"} . "/bastille-firewall-schedule";
    my $orig_file = "/bastille-firewall-schedule";

    unless ( -e $GLOBAL_PREFIX . $placed_file ) {
	&B_place($orig_file,$placed_file);
	&B_chmod(0500,$placed_file);
    }


    # Place the bastille-firewall-reset file
    my $placed_file = $GLOBAL_DIR{"sbin"} . "/bastille-firewall-reset";
    my $orig_file = "/bastille-firewall-reset";

    unless ( -e $GLOBAL_PREFIX . $placed_file ) {
	&B_place($orig_file,$placed_file);
	&B_chmod(0500,$placed_file);
    }

    if ($GLOBAL_DISTRO =~ /^RH6/) {
	
	my $ifup_file = $GLOBAL_DIR{"sbin"} . "/ifup-local";

	if ( ( -e $GLOBAL_PREFIX . $ifup_file ) and ( ! -e $GLOBAL_PREFIX . $ifup_file . ".pre-bastille")) {

	    # no pre-bastille yet; see if the current ifup-local
	    # script is ours
	    $looksLikeOurs = 0;
	    &B_open(*IFUP_CURRENT,$GLOBAL_PREFIX . $ifup_file);
	    while ($line=<IFUP_CURRENT>) {
		if ( $line =~ /\/sbin\/bastille\-firewall\-reset/ ) {
		    # it looks like our script, no need
		    # to make a pre-bastille copy
		    $looksLikeOurs = 1;
		}
	    }
	    &B_close(*IFUP_CURRENT);
	    if ( $looksLikeOurs == 0 ) {
		# make copy of current ifup-local script that our 
		# ifup-local script will call
		# ACK! I _hate_ "unless ( ... )". Why can't folks use "if (! ...)" ???
		# Gee, Larry, let's make another function that's Perl-specific. Gag!
		&B_cp($ifup_file,$ifup_file . ".pre-bastille");
#		&B_blank_file($ifup_file);
		&B_place("/ifup-local",$ifup_file);
		&B_chmod(0500,$ifup_file);
	    }
	}

	if ( ! -e $GLOBAL_PREFIX . $ifup_file ) {
	    &B_place("/ifup-local",$ifup_file);
	}
	
    }

}

1;

